/** genStuff
  * you input the name of a module (e.g. project) and the API (e.g. http://localhost/db/project)
  * it outputs a state, action types, action creators, reducer, workers and watchers
  * to handle a restful API.
  */
export default function reduxGen (name) {
  const actionTypes = {}
  const actions = {}

  const actionList = ['create', 'update', 'delete']

  /** Populate the actionTypes and actions */
  actionList.forEach(action => {
    const actionType = `${action.toUpperCase()}_${name.toUpperCase()}`
    const actionName = `${action}${name[0].toUpperCase()}${name.substring(1)}`
    actionTypes[actionType] = `${name}/${actionType}`
    actions[actionName] = (id, data) => { return { type: `${name}/${actionType}`, id, data } }
  })

  const state = []

  function reducer (state = [], action) {
    let nextState
    switch (action.type) {
      case actionTypes[`CREATE_${name.toUpperCase()}`]:
        nextState = [ ...state, action.data ]
        return nextState
      case actionTypes[`UPDATE_${name.toUpperCase()}`]:
        nextState = [ ...state.slice(0, action.id), { ...state[action.id], ...action.data }, ...state.slice(action.id + 1) ]
        return nextState
      case actionTypes[`REMOVE_${name.toUpperCase()}`]:
        nextState = [ ...state.slice(0, action.id), ...state.slice(action.id + 1) ]
        return nextState
      default:
        return state
    }
  }

  return {
    actionTypes,
    actions,
    state,
    reducer,
    watchers: null,
    workers: null
  }
}
